@@ -175,9 +175,14 @@ static void send_arp(int fd,unsigned short op,unsigned long local_ip,
   struct sockaddr_atmsvc *local_addr,unsigned long remote_ip,
   struct sockaddr_atmsvc *remote_addr)
 {
+    struct sockaddr_atmsvc local;
     struct atmarphdr *hdr;
     unsigned char *buffer,*here;
 
+    if (!local_addr) {
+	if (get_local(fd,&local) < 0) return;
+	local_addr = &local;
+    }
     buffer = alloc(MAX_ATMARP_SIZE+RFC1483LLC_LEN);
     memcpy(buffer,llc_oui_arp,RFC1483LLC_LEN);
     hdr = (struct atmarphdr *) (buffer+RFC1483LLC_LEN);
@@ -209,7 +214,7 @@ static void arp_request(ITF *itf,unsigned long ip)
 	diag(COMPONENT,DIAG_ERROR,"ARP server has no usable VCC");
 	return;
     }
-    send_arp(vcc->fd,ARPOP_REQUEST,itf->local_ip,&itf->local_addr,ip,NULL);
+    send_arp(vcc->fd,ARPOP_REQUEST,itf->local_ip,NULL,ip,NULL);
 }
 
 
@@ -217,7 +222,7 @@ static void inarp_for_itf(int fd,ITF *itf)
 {
     if (itf->local_ip) {
 	diag(COMPONENT,DIAG_DEBUG,"  on itf %d",itf->number);
-	send_arp(fd,ARPOP_InREQUEST,itf->local_ip,&itf->local_addr,0,NULL);
+	send_arp(fd,ARPOP_InREQUEST,itf->local_ip,NULL,0,NULL);
     }
 }
 
@@ -267,7 +272,7 @@ static void inarp_reply(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	diag(COMPONENT,DIAG_ERROR,"InARP request is unroutable");
 	return;
     }
-    send_arp(vcc->fd,ARPOP_InREPLY,itf->local_ip,&itf->local_addr,ip,addr);
+    send_arp(vcc->fd,ARPOP_InREPLY,itf->local_ip,NULL,ip,addr);
 }
 
 
@@ -401,7 +406,7 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	return;
     }
     entry = lookup_ip(itf,ip);
-    assert(vcc->entry);
+    assert(!vcc || vcc->entry);
     if (entry && vcc && vcc->entry->itf && entry != vcc->entry) {
 	diag(COMPONENT,DIAG_DEBUG,"collision on %d.%d.%d.%d",ipp[0],ipp[1],
 	  ipp[2],ipp[3]);
@@ -612,15 +617,13 @@ void incoming_arp(VCC *vcc,struct atmarphdr *hdr,int len)
    ipp = (unsigned char *) &src_ip;
    diag(COMPONENT,DIAG_DEBUG,"  SRC IP: %d.%d.%d.%d",ipp[0],ipp[1],ipp[2],
      ipp[3]);
-   if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &source,
-     A2T_PRETTY) >= 0)
-	diag(COMPONENT,DIAG_DEBUG,"  SRC ATM: %s",buffer);
+   if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &source,pretty)
+     >= 0) diag(COMPONENT,DIAG_DEBUG,"  SRC ATM: %s",buffer);
    ipp = (unsigned char *) &tgt_ip;
    diag(COMPONENT,DIAG_DEBUG,"  DST IP: %d.%d.%d.%d",ipp[0],ipp[1],ipp[2],
      ipp[3]);
-   if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &target,
-     A2T_PRETTY) >= 0)
-	diag(COMPONENT,DIAG_DEBUG,"  DST ATM: %s",buffer);
+   if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &target,pretty)
+     >= 0) diag(COMPONENT,DIAG_DEBUG,"  DST ATM: %s",buffer);
 }
     switch (ntohs(hdr->ar_op)) {
 	case ARPOP_REQUEST:
@@ -774,7 +777,7 @@ int arp_ioctl(int itf_num,unsigned long cmd,struct atmarpreq *req)
     switch (cmd) {
 	case SIOCSARP:
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,
-	      (struct sockaddr *) &req->arp_ha,A2T_PRETTY) < 0) {
+	      (struct sockaddr *) &req->arp_ha,pretty) < 0) {
 		diag(COMPONENT,DIAG_ERROR,"a2t fails on SIOCSARP");
 		return -EINVAL;
 	    }
