@@ -10,6 +10,7 @@
 #include <sys/types.h>
 #include <linux/atmarp.h>
 
+#include "atm.h"
 #include "atmd.h"
 
 #include "io.h"
@@ -27,23 +28,26 @@
 ITF *itfs = NULL;
 ENTRY *unknown_incoming = NULL;
 int debug;
+int pretty = A2T_PRETTY | A2T_NAME;
 
 
 static void usage(const char *name)
 {
-    fprintf(stderr,"usage: %s [ -b ] [ -d ]\n",name);
+    fprintf(stderr,"usage: %s [ -b ] [ -d ] [ -n ]\n",name);
     exit(1);
 }
 
 
 int main(int argc,char **argv)
 {
+    const char *dump_dir;
     int c,background;
 
     set_application("atmarpd");
     set_verbosity(NULL,DIAG_INFO);
+    dump_dir = DUMP_DIR;
     background = 0;
-    while ((c = getopt(argc,argv,"bdnp")) != EOF)
+    while ((c = getopt(argc,argv,"bdD:np")) != EOF)
 	switch (c) {
 	    case 'b':
 		background = 1;
@@ -52,14 +56,20 @@ int main(int argc,char **argv)
 		set_verbosity(NULL,DIAG_DEBUG);
 		debug = 1;
 		break;
-	    case 'n':
-		/* will enable NSAP matching @@@ */
+	    case 'D':
+		dump_dir = optarg;
+		break;
+	    case 'n': /* @@@ was planned for NSAP matching */
+		pretty = A2T_PRETTY;
+		break;
 	    case 'p':
 		/* paranoid anti-firewall-tunneling mode @@@ */
 	    default:
 		usage(argv[0]);
 	}
     diag(COMPONENT,DIAG_INFO,"Linux ATM ARP, version " VERSION);
+    if (chdir(dump_dir) < 0)
+	diag(COMPONENT,DIAG_ERROR,"chdir %s: %s",dump_dir,strerror(errno));
     open_all();
     if (background) {
     	pid_t pid;
