@@ -52,6 +52,7 @@ void open_all(void)
 	diag(COMPONENT,DIAG_FATAL,"socket: %s",strerror(errno));
     memset(&addr,0,sizeof(addr));
     addr.sas_family = AF_ATMSVC;
+    addr.sas_rxtp.class = ATM_UBR; /* compatible with everything else */
     addr.sas_addr.bhli.hl_type = ATM_HL_NONE;
     addr.sas_addr.blli = &blli;
     blli.l2_proto = ATM_L2_ISO8802;
@@ -226,8 +227,8 @@ static void accept_new(void)
     Q_INSERT_HEAD(unknown_incoming,entry);
     vcc->entry = entry;
     Q_INSERT_HEAD(entry->vccs,vcc);
-    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,
-      A2T_PRETTY) < 0) strcpy(buffer,"<atm2text error>");
+    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) &addr,pretty) <
+      0) strcpy(buffer,"<atm2text error>");
     diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
     incoming_call(vcc);
 }
@@ -432,14 +433,13 @@ int ip_itf_info(int number,unsigned long *ip,unsigned long *netmask)
 }
 
 
-int atm_itf_info(struct sockaddr_atmsvc *addr)
+int get_local(int fd,struct sockaddr_atmsvc *addr)
 {
-    struct atmif_sioc req;
+    int length,result;
 
-    req.number = 0; /* @@@ */
-    req.arg = addr;
-    req.length = sizeof(*addr);
-    if (ioctl(kernel,SIOCGIFATMADDR,&req) >= 0) return 0;
-    diag(COMPONENT,DIAG_FATAL,"ioctl SIOCGIFATMADDR: %s",strerror(errno));
-    return -1;
+    length = sizeof(struct sockaddr_atmsvc);
+    result = getsockname(fd,addr,&length);
+    if (result < 0)
+	diag(COMPONENT,DIAG_ERROR,"getsockname: %s",strerror(errno));
+    return result;
 }
