@@ -11,6 +11,11 @@
 #include "atmd.h"
 
 
+#define DUMP_DIR	"/var/run"
+#define DUMP_FILE	"atmarpd.table"
+#define TMP_DUMP_FILE	"~atmarpd.table"
+
+
 typedef struct _vcc {
     int connecting;
     int active; /* indicate direction - for user entertainment only */
@@ -43,7 +48,6 @@ typedef struct _entry {
 typedef struct _itf {
     unsigned long local_ip; /* @@@ */
     unsigned long netmask;
-    struct sockaddr_atmsvc local_addr; /* @@@ */
     int number;
     ENTRY *table;
     ENTRY *arp_srv; /* NULL is none */
@@ -63,11 +67,12 @@ extern const char *vcc_state_name[];
 extern ITF *itfs;
 extern ENTRY *unknown_incoming;
 
+extern int pretty;
+
 
 ENTRY *lookup_ip(const ITF *itf,unsigned long ip);
 ENTRY *lookup_addr(const ITF *itf,const struct sockaddr_atmsvc *addr);
 
-void dump_itf(ITF *itf);
 void dump_all(void);
 
 #endif
