@@ -49,15 +49,15 @@
 typedef enum IlmiState { NO_CHANGE, RESTART, NET_PREFIX, ADDRESS, POLL } IlmiState;
 
 void usage(char *name);
-void ilmi_loop(int fd);
+void ilmi_loop(int fd, int itf);
 
 void usage(char *name)
 {
-  fprintf(stderr, "usage: %s [-b] [-d]\n", name);
+  fprintf(stderr, "usage: %s [-b] [-d] <interface>\n", name);
   exit(1);
 }
 
-void ilmi_loop(int fd)
+void ilmi_loop(int fd, int itf)
 {
   int poll_misses = 0;
   long int requestID = 0;
@@ -84,7 +84,7 @@ void ilmi_loop(int fd)
 	{
 	case RESTART:
 	  deleteNetPrefix();
-	  delete_nsap();
+	  delete_nsap(itf);
 	  diag(COMPONENT, DIAG_DEBUG, "sending cold-start");
 	  diag(COMPONENT, DIAG_DEBUG, "sending get-next");
 	  send_message(fd, coldstart_message);
@@ -154,7 +154,7 @@ void ilmi_loop(int fd)
 			    if(in_message->data->a.get_response->error_status == NOERROR)
 			      {
 				diag(COMPONENT, DIAG_INFO, "ATM address registered");
-				add_nsap(netprefix_oid, esi_oid);
+				add_nsap(itf, netprefix_oid, esi_oid);
 				poll_misses = 0;
 				new_state = POLL;
 			      }
@@ -236,7 +236,7 @@ void ilmi_loop(int fd)
 
 int main(int argc, char *argv[])
 {
-  int fd, opt, bg = 0;
+  int fd, opt, itf = 0, bg = 0;
   pid_t pid;
 
   set_application("ilmid");
@@ -255,10 +255,13 @@ int main(int argc, char *argv[])
 	usage(argv[0]);
       }
 
+  if(argc == optind + 1)
+    itf = atoi(argv[optind++]);
+
   if(argc != optind)
     usage(argv[0]);
 
-  diag(COMPONENT, DIAG_INFO, "Linux ATM ILMI version 0.6");
+  diag(COMPONENT, DIAG_INFO, "Linux ATM ILMI, version %s", VERSION);
 
   if(bg)
     {
@@ -270,8 +273,8 @@ int main(int argc, char *argv[])
     }
   
   InitNibbleMem(512, 512);
-  fd = open_ilmi();
-  ilmi_loop(fd);
+  fd = open_ilmi(itf);
+  ilmi_loop(fd, itf);
   close(fd);
   return 0;
 }
