@@ -1,6 +1,6 @@
 /* diag.c - Diagnostic messages */
 
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -18,10 +18,17 @@ typedef struct _component {
 } COMPONENT;
 
 
+static const char *app_name = NULL;
 static COMPONENT *components = NULL;
 static int default_verbosity = DIAG_INFO;
 
 
+void set_application(const char *name)
+{
+    app_name = name;
+}
+
+
 void set_verbosity(const char *component,int level)
 {
     COMPONENT *walk;
@@ -50,7 +57,8 @@ void vdiag(const char *component,int severity,const char *fmt,va_list ap)
 	if (!strcmp(walk->name,component)) break;
     if (severity > (walk ? walk->verbosity : default_verbosity)) return;
     fflush(stdout);
-    fprintf(stderr,"%s: ",component);
+    if (app_name) fprintf(stderr,"%s:%s: ",app_name,component);
+    else fprintf(stderr,"%s: ",component);
     vfprintf(stderr,fmt,ap);
     fputc('\n',stderr);
     if (severity == DIAG_FATAL) {
