@@ -1,6 +1,6 @@
 /* proto.c - Common protocol functions and structures */
  
-/* Written 1995 by Werner Almesberger, EPFL-LRC */
+/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdarg.h>
@@ -14,6 +14,7 @@
 
 #include "io.h"
 #include "proto.h"
+#include "sap.h"
 
 
 #define COMPONENT "SIGD"
@@ -65,18 +66,6 @@ SOCKET *new_sock(unsigned long id)
 }
 
 
-void free_sap(struct sockaddr_atmsvc *sap)
-{
-    struct atm_blli *walk,*next;
-
-    for (walk = sap->sas_addr.blli; walk; walk = next) {
-	next = walk->next;
-	free(walk);
-    }
-    free(sap);
-}
-
-
 void free_sock(SOCKET *sock)
 {
     SOCKET **walk;
@@ -96,8 +85,8 @@ void free_sock(SOCKET *sock)
     if (sock->listen)
         diag(COMPONENT,DIAG_ERROR,"socket 0x%lx has non-empty listen queue",
 	  sock->id);
-    if (sock->local) free_sap(sock->local);
-    if (sock->remote) free_sap(sock->remote);
+    if (sock->local) free(sock->local);
+    if (sock->remote) free(sock->remote);
     sock->state = ss_invalid;
     free(sock);
 }
@@ -112,98 +101,17 @@ void new_state(SOCKET *sock,STATE state)
 }
 
 
-struct sockaddr_atmsvc *alloc_sap(const struct sockaddr_atmsvc *sap)
+SOCKET *lookup_sap(struct sockaddr_atmsvc *sap,struct sockaddr_atmsvc **res)
 {
-    struct sockaddr_atmsvc *new_sap;
-    struct atm_blli *walk,**here;
-
-    /*if (!*sap->sas_addr.prv && !*sap->sas_addr.pub) return NULL; @@@ */
-    new_sap = alloc_t(struct sockaddr_atmsvc);
-    *new_sap = *sap;
-    here = &new_sap->sas_addr.blli;
-    for (walk = sap->sas_addr.blli; walk; walk = walk->next) {
-	*here = alloc_t(struct atm_blli);
-	**here = *walk;
-    }
-    return new_sap;
-}
-
-
-static int match_blli(struct atm_blli *a,struct atm_blli *b)
-{
-    if (a->l2_proto != b->l2_proto || a->l3_proto != b->l3_proto) return 0;
-    switch (a->l3_proto) {
-	case ATM_L3_TR9577:
-	    if (a->l3.tr9577.ipi != b->l3.tr9577.ipi) return 0;
-	    if (a->l3.tr9577.ipi == NLPID_IEEE802_1_SNAP)
-		if (memcmp(a->l3.tr9577.snap,b->l3.tr9577.snap,5)) return 0;
-	    break;
-	case ATM_L3_USER:
-	    if (a->l3.user != b->l3.user) return 0;
-	    break;
-	default:
-    }
-    return 1;
-}
-
-
-static int match_sap(struct sockaddr_atmsvc *a,struct sockaddr_atmsvc *b)
-{
-    struct atm_blli *a_blli,*b_blli;
-    int length;
-
-/* @@@ check AAL */
-    if (a->sas_addr.bhli.hl_type != b->sas_addr.bhli.hl_type) return 0;
-    switch (a->sas_addr.bhli.hl_type) {
-	case ATM_HL_ISO:
-	case ATM_HL_USER:
-	    length = a->sas_addr.bhli.hl_length;
-	    if (length != b->sas_addr.bhli.hl_length) return 0;
-	    break;
-#ifdef UNI30
-	case ATM_HL_HLP:
-	    length = 4;
-	    break;
-#endif
-	case ATM_HL_VENDOR:
-	    length = 7;
-	    break;
-	default:
-	    length = 0;
-    }
-    if (length && memcmp(a->sas_addr.bhli.hl_info,b->sas_addr.bhli.hl_info,
-      length)) return 0;
-    a_blli = a->sas_addr.blli;
-    b_blli= b->sas_addr.blli;
-    if (!a_blli && !b_blli) return 1; /* redundant */
-    while (a_blli) {
-	while (b_blli) {
-	    if (match_blli(a_blli,b_blli)) return 1;
-	    b_blli = b_blli->next;
-	}
-	a_blli = a_blli->next;
-    }
-    return 0;
-}
-
-
-SOCKET *lookup_sap(struct sockaddr_atmsvc *sap)
-{
-    SOCKET *walk;
+    SOCKET *walk,*wildcard;
 
+    wildcard = NULL;
     for (walk = sockets; walk; walk = walk->next)
 	if (walk->state == ss_listening) {
-	    if (!walk->local) break;
-	    if (sap->sas_txtp.max_sdu && sap->sas_txtp.max_sdu <
-	      walk->local->sas_txtp.max_sdu) continue;
-	    if (walk->local->sas_rxtp.max_sdu && sap->sas_rxtp.max_sdu >
-	      walk->local->sas_rxtp.max_sdu) continue;
-	    /* check PCR compat for CBR @@@ */
-	    if (!walk->local->sas_addr.bhli.hl_type &&
-	      !walk->local->sas_addr.blli) break;
-	    else if (match_sap(walk->local,sap)) break;
+	    if (walk->local && sap_compat(walk->local,sap,res)) return walk;
+	    if (!walk->local) wildcard = walk;
 	}
-    return walk;
+    return wildcard;
 }
 
 
@@ -247,26 +155,28 @@ const char *mid2name(unsigned char mid)
 
 
 void send_kernel(unsigned long vcc,unsigned long listen_vcc,
-  enum atmsvc_msg_type type,int reply,SOCKET *sock)
+  enum atmsvc_msg_type type,int reply,struct sockaddr_atmpvc *pvc,
+  struct sockaddr_atmsvc *svc)
 {
     struct atmsvc_msg *msg;
     struct atm_blli *walk;
     int blli,i;
 
     blli = 0;
-    for (walk = sock && sock->remote ? sock->remote->sas_addr.blli : NULL;
-      walk; walk = walk->next) blli++;
+    for (walk = svc ? svc->sas_addr.blli : NULL; walk; walk = walk->next)
+	blli++;
     msg = alloc(bllis2msg(blli));
     msg->vcc = vcc;
     msg->listen_vcc = listen_vcc;
     msg->type = type;
     msg->reply = reply;
-    memset(&msg->remote,0,sizeof(msg->remote));
-    if (sock) msg->pvc = sock->pvc;
-    if (sock && sock->remote) {
-	msg->remote = *sock->remote;
+    if (pvc) msg->pvc = *pvc;
+    else memset(&msg->pvc,0,sizeof(msg->pvc));
+    if (!svc) memset(&msg->svc,0,sizeof(msg->svc));
+    else {
+	msg->svc = *svc;
 	i = 0;
-	for (walk = sock->remote->sas_addr.blli; walk; walk = walk->next)
+	for (walk = svc->sas_addr.blli; walk; walk = walk->next)
 	    msg->blli[i++] = *walk;
     }
     to_kernel(msg);
@@ -296,8 +206,20 @@ void send_release(SOCKET *sock,unsigned char reason,...)
 	default:
     }
     va_end(ap);
-    size = q_close(&dsc);
-    to_signaling(q_buffer,size);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
+}
+
+
+void send_release_complete(unsigned long call_ref,unsigned char cause)
+{
+    Q_DSC dsc;
+    int size;
+ 
+    q_create(&dsc,q_buffer,MAX_Q_MSG);
+    q_assign(&dsc,QF_msg_type,QMSG_REL_COMP);
+    q_assign(&dsc,QF_call_ref,call_ref);
+    if (cause) q_assign(&dsc,QF_cause,cause); /* @@@ more data */
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
 }
 
 
@@ -310,7 +232,7 @@ void set_error(SOCKET *sock,int code)
 void send_close(SOCKET *sock)
 {
     if (sock->error == 1234) diag(COMPONENT,DIAG_ERROR,"BUG! BUG! BUG!");
-    send_kernel(sock->id,0,as_close,sock->error,NULL);
+    SEND_ERROR(sock->id,sock->error);
     sock->error = 1234;
 }
 
@@ -323,3 +245,53 @@ void q_report(int severity,const char *msg,...)
     vdiag("QMSG",severity,msg,ap);
     va_end(ap);
 }
+
+
+typedef struct _vpci {
+    int vpci;
+    int itf;
+    struct _vpci *next;
+} VPCI;
+
+
+static VPCI *vpcis = NULL;
+
+
+void enter_vpci(int vpci,int itf)
+{
+    VPCI *entry;
+
+    for (entry = vpcis; entry; entry = entry->next)
+	if (entry->vpci == vpci) {
+	    diag(COMPONENT,DIAG_ERROR,"ignoring duplicate VPCI %d (itf %d)",
+	      vpci,itf);
+	    return;
+	}
+    entry = alloc_t(VPCI);
+    entry->vpci = vpci;
+    entry->itf = itf;
+    entry->next = vpcis;
+    vpcis = entry;
+}
+
+
+int get_itf(int *vpci)
+{
+    VPCI *best,*walk;
+
+    best = NULL;
+    for (walk = vpcis; walk; walk = walk->next)
+	if (walk->vpci <= *vpci && (!best || best->vpci < walk->vpci))
+	    best = walk;
+    if (!best) return signaling_pvc.sap_addr.itf;
+    *vpci -= best->vpci;
+    return best->itf;
+}
+
+
+void init_addr(void)
+{
+    VPCI *walk;
+
+    for (walk = vpcis; walk; walk = walk->next) itf_load(walk->itf);
+}
