@@ -54,19 +54,23 @@ extern SOCKET *sockets;
 extern unsigned char q_buffer[];
 
 extern int net;
-extern short sig_itf;
+extern int pretty;
 
 
 #define msg2bllis(s) (((int) (s)-(int) sizeof(struct atmsvc_msg))/ \
   (int) sizeof(struct atm_blli)+1)
 #define bllis2msg(n) (sizeof(struct atmsvc_msg)+sizeof(struct atm_blli)*((n)-1))
 
+#define SEND_ERROR(vcc,code) send_kernel(vcc,0L,as_close,code,NULL,NULL)
+
 
 void from_kernel(struct atmsvc_msg *msg,int size);
+void itf_load(int itf);
 
 void to_q2931(void *msg,int size);
 void send_kernel(unsigned long vcc,unsigned long listen_vcc,
-  enum atmsvc_msg_type type,int reply,SOCKET *sock);
+  enum atmsvc_msg_type type,int reply,struct sockaddr_atmpvc *pvc,
+  struct sockaddr_atmsvc *svc);
 void from_net(void *msg,int size);
 void to_signaling(void *msg,int size);
 void saal_failure(void);
@@ -76,14 +80,17 @@ void clear_all_calls_on_T309(void);
 SOCKET *new_sock(unsigned long id);
 void free_sock(SOCKET *sock);
 void new_state(SOCKET *sock,STATE state);
-struct sockaddr_atmsvc *alloc_sap(const struct sockaddr_atmsvc *sap);
-void free_sap(struct sockaddr_atmsvc *sap);
-SOCKET *lookup_sap(struct sockaddr_atmsvc *sap);
+SOCKET *lookup_sap(struct sockaddr_atmsvc *sap,struct sockaddr_atmsvc **res);
 void send_release(SOCKET *sock,unsigned char reason,...);
+void send_release_complete(unsigned long call_ref,unsigned char cause);
 
 const char *mid2name(unsigned char mid);
 
 void set_error(SOCKET *sock,int code);
 void send_close(SOCKET *sock);
 
+void enter_vpci(int vpci,int itf);
+int get_itf(int *vpci);
+void init_addr(void);
+
 #endif
