@@ -4,6 +4,7 @@
  
 
 #include <stdarg.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
@@ -14,6 +15,7 @@
 #include "qlib.h"
 
 #include "proto.h"
+#include "sap.h"
 #include "io.h"
 #include "timeout.h"
 
@@ -44,132 +46,28 @@ static void send_call_proceeding(SOCKET *sock)
     if (net) {
 	q_assign(&dsc,QF_vpi,0);
 	q_assign(&dsc,QF_vci,assign_vci); /* @@@ */
-	sock->pvc.sap_addr.itf = sig_itf; /* @@@ use VPCI */
+	sock->pvc.sap_addr.itf = signaling_pvc.sap_addr.itf;
 	sock->pvc.sap_addr.vpi = 0;
 	sock->pvc.sap_addr.vci = assign_vci++;
     }
-    size = q_close(&dsc);
-    to_signaling(q_buffer,size);
-}
-
-
-static void send_release_complete(unsigned long call_ref,unsigned char cause)
-{
-    Q_DSC dsc;
-    int size;
-
-    q_create(&dsc,q_buffer,MAX_Q_MSG);
-    q_assign(&dsc,QF_msg_type,QMSG_REL_COMP);
-    q_assign(&dsc,QF_call_ref,call_ref);
-    /* set cause (only if !!cause) */
-    size = q_close(&dsc);
-    to_signaling(q_buffer,size);
-}
-
-
-static struct sockaddr_atmsvc *build_sap(void)
-{
-    struct sockaddr_atmsvc *sap;
-    struct atm_blli *blli;
-
-    sap = alloc_t(struct sockaddr_atmsvc);
-    memset(sap,0,sizeof(struct sockaddr_atmsvc));
-    if (q_present(&in_dsc,QF_aal_type))
-	if (q_fetch(&in_dsc,QF_aal_type) != 5)
-	    diag(COMPONENT,DIAG_ERROR,"AAL type %d requested",
-	      q_fetch(&in_dsc,QF_aal_type));
-    if (q_present(&in_dsc,QF_best_effort))
-	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_UBR;
-    else {
-	sap->sas_txtp.class = sap->sas_rxtp.class = ATM_CBR;
-	if (q_present(&in_dsc,QF_fw_pcr_01))
-	    sap->sas_txtp.min_pcr = sap->sas_txtp.max_pcr =
-	      q_fetch(&in_dsc,QF_fw_pcr_01);
-	if (q_present(&in_dsc,QF_bw_pcr_01))
-	    sap->sas_rxtp.min_pcr = sap->sas_rxtp.max_pcr =
-	      q_fetch(&in_dsc,QF_bw_pcr_01);
-	/* SHOULD ... fail call if anything is missing ... @@@ */
-    }
-    if (q_present(&in_dsc,QF_fw_max_sdu))
-	sap->sas_txtp.max_sdu = q_fetch(&in_dsc,QF_fw_max_sdu);
-    if (q_present(&in_dsc,QF_bw_max_sdu))
-	sap->sas_rxtp.max_sdu = q_fetch(&in_dsc,QF_bw_max_sdu);
-    if (q_present(&in_dsc,QG_bhli)) {
-	sap->sas_addr.bhli.hl_type = q_fetch(&in_dsc,QF_hli_type);
-	switch (sap->sas_addr.bhli.hl_type) {
-	    case ATM_HL_ISO:
-		sap->sas_addr.bhli.hl_length = q_length(&in_dsc,QF_iso_hli);
-		q_read(&in_dsc,QF_iso_hli,sap->sas_addr.bhli.hl_info,
-		  sap->sas_addr.bhli.hl_length);
-		break;
-	    case ATM_HL_USER:
-		sap->sas_addr.bhli.hl_length = q_length(&in_dsc,QF_user_hli);
-		q_read(&in_dsc,QF_user_hli,sap->sas_addr.bhli.hl_info,
-		  sap->sas_addr.bhli.hl_length);
-		break;
-#ifdef UNI30
-	    case ATM_HL_HLP:
-		sap->sas_addr.bhli.hl_length = 4;
-		q_read(&in_dsc,QF_hlp,sap->sas_addr.bhli.hl_info,4);
-		break;
-#endif
-	    case ATM_HL_VENDOR:
-		sap->sas_addr.bhli.hl_length = 7;
-		q_read(&in_dsc,QF_hli_oui,sap->sas_addr.bhli.hl_info,3);
-		q_read(&in_dsc,QF_app_id,sap->sas_addr.bhli.hl_info+3,4);
-		break;
-	    default:
-		diag(COMPONENT,DIAG_FATAL,"unrecognized hl_type");
-	}
-    }
-
-#define GET(var,field) \
-   ({ if (q_present(&in_dsc,field)) blli->var = q_fetch(&in_dsc,field); })
-
-    if (q_present(&in_dsc,QG_blli)) {
-	sap->sas_addr.blli = blli = alloc_t(struct atm_blli);
-	if (q_present(&in_dsc,QF_uil2_proto)) {
-	    blli->l2_proto = q_fetch(&in_dsc,QF_uil2_proto);
-	    GET(l2.itu.mode,QF_l2_mode);
-	    GET(l2.itu.window,QF_window_size);
-	    GET(l2.user,QF_user_l2);
-	}
-	if (q_present(&in_dsc,QF_uil3_proto)) {
-	    blli->l3_proto = q_fetch(&in_dsc,QF_uil3_proto);
-	    GET(l3.itu.mode,QF_l3_mode);
-	    GET(l3.itu.def_size,QF_def_pck_size);
-	    GET(l3.itu.window,QF_pck_win_size);
-	    GET(l3.user,QF_user_l3);
-	    if (q_present(&in_dsc,QF_ipi_high)) {
-		blli->l3.tr9577.ipi = q_fetch(&in_dsc,QF_ipi_high) << 1;
-		if (blli->l3.tr9577.ipi != NLPID_IEEE802_1_SNAP)
-		    blli->l3.tr9577.ipi |= q_fetch(&in_dsc,QF_ipi_low);
-		else if (!q_present(&in_dsc,QF_oui)) blli->l3.tr9577.ipi |= 1;
-		    else {
-			q_read(&in_dsc,QF_oui,blli->l3.tr9577.snap,3);
-			q_read(&in_dsc,QF_pid,blli->l3.tr9577.snap+3,2);
-		    }
-	    }
-	}
-	blli->next = NULL;
-    }
-#undef GET
-    return sap;
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
 }
 
 
 static void setup_call(unsigned long call_ref)
 {
     SOCKET *sock,*this,**walk;
-    struct sockaddr_atmsvc *sap;
+    struct sockaddr_atmsvc *sap,*new_sap;
     unsigned char buf[ATM_ESA_LEN];
-    int len,i;
+    int len,i,vpci;
 
-    sap = build_sap();
-    sock = lookup_sap(sap);
+    sap = sap_decode(&in_dsc);
+    new_sap = NULL;
+    sock = lookup_sap(sap,&new_sap);
+    if (!sock || new_sap) free(sap);
+    else new_sap = sap; /* @@@ should strip extra BLLIs */
     if (!sock) {
 	send_release_complete(call_ref,ATM_CV_INCOMP_DEST); /* @@@ dunno */
-	free_sap(sap);
 	return;
     }
     this = new_sock(0);
@@ -177,9 +75,7 @@ static void setup_call(unsigned long call_ref)
     this->q2931_state = qs_in_proc;
     this->call_ref = call_ref;
     send_call_proceeding(this);
-    this->local = sap;
-    this->pvc.sap_txtp = sap->sas_txtp;
-    this->pvc.sap_rxtp = sap->sas_rxtp;
+    this->local = new_sap;
     /* if (sock->local) *this->local->sas_addr = sock->local->sas_addr; ??? */
     diag(COMPONENT,DIAG_DEBUG,"AAL type %ld",q_fetch(&in_dsc,QF_aal_type));
     len = q_read(&in_dsc,QF_cdpn_esa,(void *) &buf,sizeof(buf));
@@ -188,29 +84,28 @@ static void setup_call(unsigned long call_ref)
 	for (i = 0; i < (len < 30 ? len : 30); i++) printf(" %02X",buf[i]);
 	putchar('\n');
     }
-    if (q_present(&in_dsc,QF_vci)) {
-	this->pvc.sap_addr.itf = sig_itf; /* @@@ use VPCI */
-	this->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
-	this->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
-	diag(COMPONENT,DIAG_DEBUG,"VPI/VCI: %d/%d",this->pvc.sap_addr.vpi,
-	  this->pvc.sap_addr.vci);
-    }
+    vpci = q_fetch(&in_dsc,QF_vpi);
+    this->pvc.sap_addr.itf = get_itf(&vpci);
+    this->pvc.sap_addr.vpi = vpci;
+    this->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
+    diag(COMPONENT,DIAG_DEBUG,"ITF.VPI.VCI: %d.%d.%d",this->pvc.sap_addr.itf,
+      this->pvc.sap_addr.vpi,this->pvc.sap_addr.vci);
+    this->remote = alloc_t(struct sockaddr_atmsvc);
+    memset(this->remote,0,sizeof(struct sockaddr_atmsvc));
+    this->pvc.sap_txtp = this->remote->sas_txtp = new_sap->sas_txtp;
+    this->pvc.sap_rxtp = this->remote->sas_rxtp = new_sap->sas_rxtp;
     if (q_present(&in_dsc,QF_cgpn)) { /* should handle E.164 too */
 	char buffer[MAX_ATM_ADDR_LEN+1];
 
-	this->remote = alloc_t(struct sockaddr_atmsvc);
-	memset(this->remote,0,sizeof(struct sockaddr_atmsvc));
 	this->remote->sas_family = AF_ATMSVC;
 	i = q_read(&in_dsc,QF_cgpn,(void *) this->remote->sas_addr.prv,
 	  ATM_ESA_LEN);
-	this->remote->sas_txtp = sap->sas_txtp;
-	this->remote->sas_rxtp = sap->sas_rxtp;
 	/*printf("addr_len = %d\n",i);*/
 	if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *) this->remote,
-	  A2T_PRETTY) < 0) strcpy(buffer,"<invalid address>");
+	  pretty) < 0) strcpy(buffer,"<invalid address>");
 	diag(COMPONENT,DIAG_DEBUG,"Incoming call from %s",buffer);
     }
-    send_kernel(0,sock->id,as_indicate,0,this);
+    send_kernel(0,sock->id,as_indicate,0,NULL,this->remote);
     for (walk = &sock->listen; *walk; walk = &(*walk)->listen);
     *walk = this;
     diag(COMPONENT,DIAG_DEBUG,"SE vpi.vci=%d.%d",this->pvc.sap_addr.vpi,
@@ -253,8 +148,7 @@ static void send_status(SOCKET *sock,unsigned char cause,...)
 	default:
     }
     va_end(ap);
-    size = q_close(&dsc);
-    to_signaling(q_buffer,size);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
 }
 
 
@@ -266,8 +160,7 @@ static void send_status_enq(SOCKET *sock)
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     q_assign(&dsc,QF_msg_type,QMSG_STATUS_ENQ);
     q_assign(&dsc,QF_call_ref,sock->call_ref);
-    size = q_close(&dsc);
-    to_signaling(q_buffer,size);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
     /* @@@ should start T322 */
 }
 
@@ -280,8 +173,7 @@ static void send_connect_ack(SOCKET *sock)
     q_create(&dsc,q_buffer,MAX_Q_MSG);
     q_assign(&dsc,QF_msg_type,QMSG_CONN_ACK);
     q_assign(&dsc,QF_call_ref,sock->call_ref);
-    size = q_close(&dsc);
-    to_signaling(q_buffer,size);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
 }
 
 
@@ -299,8 +191,7 @@ static void send_restart_ack(int vpi,int vci)
 	q_assign(&dsc,QF_vpi,vpi);
 	q_assign(&dsc,QF_vci,vci);
     }
-    size = q_close(&dsc);
-    to_signaling(q_buffer,size);
+    if ((size = q_close(&dsc)) >= 0) to_signaling(q_buffer,size);
 }
 
 
@@ -338,11 +229,15 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    /* check for 2nd CALL_PROC @@@ */
 	    STOP_TIMER(sock);
 	    if (q_present(&in_dsc,QG_conn_id)) {
-		sock->pvc.sap_addr.itf = sig_itf; /* @@@ should use VPCI */
-		sock->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
+		int vpci;
+
+		vpci = q_fetch(&in_dsc,QF_vpi);
+		sock->pvc.sap_addr.itf = get_itf(&vpci);
+		sock->pvc.sap_addr.vpi = vpci;
 		sock->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
-		diag(COMPONENT,DIAG_DEBUG,"VPI/VCI: %d/%d",
-		  sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
+		diag(COMPONENT,DIAG_DEBUG,"ITF.VPI.VCI: %d.%d.%d",
+		  sock->pvc.sap_addr.itf,sock->pvc.sap_addr.vpi,
+		  sock->pvc.sap_addr.vci);
 	    }
 	    START_TIMER(sock,T310);
 	    sock->q2931_state = qs_out_proc;
@@ -355,11 +250,15 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    if (sock->state != ss_connecting) break;
 	    STOP_TIMER(sock);
 	    if (q_present(&in_dsc,QG_conn_id)) {
-		sock->pvc.sap_addr.itf = sig_itf; /* @@@ should use VPCI */
-		sock->pvc.sap_addr.vpi = q_fetch(&in_dsc,QF_vpi);
+		int vpci;
+
+		vpci = q_fetch(&in_dsc,QF_vpi);
+		sock->pvc.sap_addr.itf = get_itf(&vpci);
+		sock->pvc.sap_addr.vpi = vpci;
 		sock->pvc.sap_addr.vci = q_fetch(&in_dsc,QF_vci);
-		diag(COMPONENT,DIAG_DEBUG,"VPI/VCI: %d/%d",
-		  sock->pvc.sap_addr.vpi,sock->pvc.sap_addr.vci);
+		diag(COMPONENT,DIAG_DEBUG,"ITF.VPI.VCI: %d/%d.%d",
+		  sock->pvc.sap_addr.itf,sock->pvc.sap_addr.vpi,
+		  sock->pvc.sap_addr.vci);
 	    }
 	    error = 0;
 	    if (!sock->pvc.sap_addr.vpi && !sock->pvc.sap_addr.vci)
@@ -375,7 +274,7 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    send_connect_ack(sock);
 	    /* @@@ fill in sock->remote */
 	    /* @@@ fill in traffic parameters */
-	    send_kernel(sock->id,0,as_okay,0,sock);
+	    send_kernel(sock->id,0,as_okay,0,&sock->pvc,NULL);
 	    new_state(sock,ss_connected);
 	    diag(COMPONENT,DIAG_INFO,"Active open succeeded (CR 0x%06X)",
 	      sock->call_ref);
@@ -389,8 +288,7 @@ static void q2931_call(SOCKET *sock,unsigned char mid)
 	    }
 	    if (sock->state != ss_accepting) break;
 	    STOP_TIMER(sock);
-	    send_kernel(sock->id,0,as_okay,0,sock);
-		/* repeat remote information - just for fun */
+	    send_kernel(sock->id,0,as_okay,0,NULL,NULL);
 	    new_state(sock,ss_connected);
 	    diag(COMPONENT,DIAG_INFO,"Passive open succeeded (CR 0x%06X)",
 	      sock->call_ref);
@@ -617,5 +515,6 @@ void to_q2931(void *msg,int size)
 	return;
     }
     q2931_call(curr,mid);
-    q_close(&in_dsc);
+    if (q_close(&in_dsc) < 0)
+	diag(COMPONENT,DIAG_ERROR,"q_close returned <0 in to_q2931");
 }
